// ***************************************************************************
// ***************************************************************************
// Copyright 2011(c) Analog Devices, Inc.
// 
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//     - Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     - Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in
//       the documentation and/or other materials provided with the
//       distribution.
//     - Neither the name of Analog Devices, Inc. nor the names of its
//       contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//     - The use of this software may or may not infringe the patent rights
//       of one or more patent holders.  This license does not release you
//       from the requirement that you obtain separate licenses from these
//       patent holders to use this software.
//     - Use of the software either in source or binary form, must be run
//       on or directly connected to an Analog Devices Inc. component.
//    
// THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED.
//
// IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, INTELLECTUAL PROPERTY
// RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// ***************************************************************************
// ***************************************************************************
// ***************************************************************************
// ***************************************************************************
// This is the LVDS/DDR interface

`timescale 1ns/100ps

module cf_adc_if (

  // jesd interface (ref_clk is the sample clock, rx_clk is ref_clk/2)

  rx_clk,
  rx_data0,
  rx_data1,
  ref_clk,

  // adc data output

  adc_valid,
  adc_data,

  // adc status

  adc_or,
  adc_pn_oos,
  adc_pn_err,

  // processor interface

  up_pn_type,
  up_data_sel,

  // debug interface (chipscope)

  adc_mon_valid,
  adc_mon_data);

  // jesd interface (ref_clk is the sample clock, rx_clk is ref_clk/2)

  input           rx_clk;
  input   [31:0]  rx_data0;
  input   [31:0]  rx_data1;
  input           ref_clk;

  // adc data output

  output          adc_valid;
  output  [63:0]  adc_data;

  // adc status

  output  [ 1:0]  adc_or;
  output  [ 1:0]  adc_pn_oos;
  output  [ 1:0]  adc_pn_err;

  // processor interface

  input   [ 1:0]  up_pn_type;
  input           up_data_sel;

  // debug interface (chipscope)

  output          adc_mon_valid;
  output  [31:0]  adc_mon_data;

  reg     [ 6:0]  adc_raddr = 'd0;
  reg     [63:0]  adc_rdata = 'd0;
  reg             adc_valid = 'd0;
  reg     [63:0]  adc_data = 'd0;
  reg     [15:0]  adc_data0 = 'd0;
  reg     [15:0]  adc_data1 = 'd0;
  reg     [ 5:0]  adc_waddr = 'd0;
  reg     [63:0]  adc_wdata = 'd0;
  reg     [ 1:0]  adc_or = 'd0;

  wire    [63:0]  adc_rdata_s;

  // bit select

  function [15:0] dalign;
    input [15:0] din;
    reg   [15:0] dout;
    begin
      dout = {din[9:8], din[7:0], din[15:10]};
      dalign = dout;
    end
  endfunction

  // debug signals

  assign adc_mon_valid = 1'b1;
  assign adc_mon_data = {adc_data1, adc_data0};

  // adc data output from this module is always at the sampling frequency.
  // adc data is received on xilinx's jesd core clock (which, in this case
  // runs at sampling clock/2, this data is transferred to the ref clock
  // domain, as the ref clock runs at sampling frequency.

  always @(posedge ref_clk) begin
    adc_raddr <= adc_raddr + 1'b1;
    adc_rdata <= adc_rdata_s;
    adc_valid <= ~adc_valid;
    adc_data <= adc_rdata[63:0];
    adc_data0 <= (adc_raddr[0] == 1'b1) ? adc_rdata[47:32] : adc_rdata[15: 0];
    adc_data1 <= (adc_raddr[0] == 1'b1) ? adc_rdata[63:48] : adc_rdata[31:16]; 
  end

  always @(posedge rx_clk) begin
    adc_waddr <= adc_waddr + 1'b1;
    adc_wdata[63:48] <= dalign(rx_data1[31:16]); 
    adc_wdata[47:32] <= dalign(rx_data0[31:16]); 
    adc_wdata[31:16] <= dalign(rx_data1[15: 0]);
    adc_wdata[15: 0] <= dalign(rx_data0[15: 0]);
    adc_or <= 'd0;
  end

  // pn monitors

  cf_pnmon i_pnmon1 (
    .adc_clk (ref_clk),
    .adc_data (adc_data1[13:0]),
    .adc_pn_oos (adc_pn_oos[1]),
    .adc_pn_err (adc_pn_err[1]),
    .up_pn_type (up_pn_type[1]));

  cf_pnmon i_pnmon0 (
    .adc_clk (ref_clk),
    .adc_data (adc_data0[13:0]),
    .adc_pn_oos (adc_pn_oos[0]),
    .adc_pn_err (adc_pn_err[0]),
    .up_pn_type (up_pn_type[0]));

  // transfer data to the reference clock side (2x)

  cf_mem #(.DW(64), .AW(6)) i_mem (
    .clka (rx_clk),
    .wea (1'b1),
    .addra (adc_waddr),
    .dina (adc_wdata),
    .clkb (ref_clk),
    .addrb (adc_raddr[6:1]),
    .doutb (adc_rdata_s));

endmodule

// ***************************************************************************
// ***************************************************************************

